#include "gasgui.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#ifndef ROCKCFG_ID
#  define ROCKCFG_ID "1.7-x86-reference"
#endif

char * config    = ROCKCFG_ID;
char * targetdir = "/mnt/target";
char * sourcedev = "/dev/cdroms/cdrom0";
char * sourcedir = "/mnt/cdrom0";
char * sourceopt = "ro";

char * mine_command = "mine";

void help()
{
	fprintf(stderr,
"\n"
"G.A.S. - GEM Autoinstall Shell\n"
"\n"
"Usage: gas [options]\n"
"\n"
"  -c " ROCKCFG_ID "\n"
"		ROCK Linux Build Config-ID.\n"
"\n"
"  -t /mnt/target\n"
"		The root directory of the target system.\n"
"\n"
"  -d /dev/cdroms/cdrom0\n"
"		The source device file.\n"
"\n"
"  -s /mnt/cdrom0\n"
"		The mount point for the source device file.\n"
"\n"
"  -o ro\n"
"		The mount options for mounting the source device file.\n"
"\n"
	);
}

void remove_package(struct package *p, int just_print_commands)
{
	char command[1024];

	snprintf(command, 1024, "%s -r -R %s %s",
		mine_command, targetdir, p->name);

	if ( just_print_commands ) {
		printf("%s\n", command);
		fflush(stdout);
	} else {
		printf("Removing %s ...\n", p->name);
		fflush(stdout);
		system(command);
	}
}

char *getgemfile(struct package *p)
{
	static char gemfile[1024];
	char command[1024];

	while (1) {
		snprintf(gemfile, 1024, "%s/%s/pkgs/%s-%s.gem",
			sourcedir, config, p->name, p->version);
		if ( !access(gemfile, F_OK) ) return gemfile;

		snprintf(gemfile, 1024, "%s/%s/pkgs/%s.gem",
			sourcedir, config, p->name);
		if ( !access(gemfile, F_OK) ) return gemfile;

		snprintf(command, 1024, "umount -v %s", sourcedir);
		system(command);

		printf("Not found on current disk: %s\n\n"
" ########################################################################\n"
" ##                                                                    ##\n"
" ##  Please insert Disk #%-2d and press ENTER                            ##\n"
" ##                                                                    ##\n"
" ########################################################################\n",
			p->name, p->disk_number);
		fflush(stdout);
		fgets(command, 1024, stdin);

		snprintf(gemfile, 1024, "%s/%s/pkgs/%s-%s.gem",
			sourcedir, config, p->name, p->version);
		if ( !access(gemfile, F_OK) ) return gemfile;

		snprintf(gemfile, 1024, "%s/%s/pkgs/%s.gem",
			sourcedir, config, p->name);
		if ( !access(gemfile, F_OK) ) return gemfile;

		snprintf(command, 1024, "mount -v -o %s %s %s",
			sourceopt, sourcedev, sourcedir);
		system(command);
	}
}

void install_package(struct package *p, int just_print_commands)
{
	char command[1024];
	char * gemfile = getgemfile(p);

	snprintf(command, 1024, "%s -i -R %s %s",
		mine_command, targetdir, gemfile);

	if ( just_print_commands ) {
		printf("%s\n", command);
		fflush(stdout);
	} else {
		printf("Disk #%d: Installing %s-%s ...\n",
		       p->disk_number, p->name, p->version);
		fflush(stdout);
		system(command);
	}
}

int main(int argc, char ** argv)
{
	int just_dump_tree = 0;
	int just_print_commands = 0;
	struct package *p;
	int opt, disk;

	while ( (opt = getopt(argc, argv, "c:t:d:s:o:DTF")) != -1 ) {
		switch (opt) {
			case 'c':
				config = optarg;
				break;
			case 't':
				targetdir = optarg;
				break;
			case 'd':
				sourcedev = optarg;
				break;
			case 's':
				sourcedir = optarg;
				break;
			case 'o':
				sourceopt = optarg;
				break;
			case 'D':
				just_dump_tree = 1;
				break;
			case 'T':
				just_print_commands = 1;
				break;
			case 'F':
				mine_command = "mine -f";
				break;
			default:
				help();
				return 0;
		}
	}

	if ( ! read_packages() ) {
		if ( just_dump_tree ) {
			print_dirtree(rootdir, 0);
			return 0;
		}
		do_selection();

		for (p = packages; p != NULL; p = p->next)
			if ( p->is_installed && !p->will_be_installed )
				remove_package(p, just_print_commands);

		for (disk = 1; disk <= max_disk_number; disk++)
		  for (p = packages; p != NULL; p = p->next)
			if ( p->disk_number == disk &&
			     !p->is_installed && p->will_be_installed )
				install_package(p, just_print_commands);
	}
	return 0;
}

