
/*
 * The code here reads all package desc files and creates the package-tree
 * in the memory. You should start reading this at function read_packages().
 */

#include "gasgui.h"
#include "memdb.h"
#include <dirent.h>
#include <limits.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

struct directory rootdir_data = { "", 1, NULL, NULL };

struct package * packages = NULL;
struct directory * directories = &rootdir_data;
struct directory * rootdir = &rootdir_data;

int directory_entry_count = 0;
int max_disk_number = 0;

struct memdb_t files_on_disks;

struct directory *
get_create_directory(struct directory * base, char *name)
{
	struct directory_entry *rc;
	int len;

	if (strchr(name, '/') == NULL) len = strlen(name);
	else len = strchr(name, '/') - name;

	for (rc = base->list; rc != NULL; rc = rc->next)
		if ( rc->content_is_subdir &&
		     !strncmp(rc->content.dir->name, name, len) &&
		     !rc->content.dir->name[len]) {
			if ( name[len] ) return get_create_directory(
						rc->content.dir, name+len+1);
			return rc->content.dir;
		}

	rc = calloc(1, sizeof(struct directory_entry));
	rc->next = base->list; base->list = rc;
	directory_entry_count++;

	rc->content_is_subdir = 1;
	rc->content.dir = calloc(1, sizeof(struct directory));
	rc->content.dir->name = malloc(len+1);
	strncpy(rc->content.dir->name, name, len);
	rc->content.dir->name[len]=0;

	rc->content.dir->next = directories;
	directories = rc->content.dir;

	if ( name[len] )
		return get_create_directory(rc->content.dir, name+len+1);
	return rc->content.dir;
}

void print_dirtree(struct directory * dir, int blanks)
{
	struct directory_entry * e;

	for (e = dir->list; e != NULL; e = e->next) {
		if ( e->content_is_subdir ) {
			printf("%*s=== %s/\n",
				blanks, "", e->content.dir->name);
			print_dirtree(e->content.dir, blanks+3);
		} else {
			printf("%*s[%c] %s - %s (%d)\n", blanks, "",
				e->content.pkg->is_installed ? '*' : ' ',
				e->content.pkg->name, e->content.pkg->version,
				e->content.pkg->disk_number);
		}
	}
}

int read_package(char *packagename, char *filename, int is_installed)
{
	struct package *p;
	struct directory *d;
	struct directory_entry *e;
	char line[160], *t;
	int line_length;
	FILE *f;

	f = fopen(filename, "r");
	if ( f == NULL ) {
		fprintf(stderr, "Can't open %s: %s\n",
		        filename, strerror(errno));
		return -1;
	}

	p = calloc(1, sizeof(struct package));
	p->next = packages; packages = p;
	p->name = packagename; p->version="0000";
	
	d = get_create_directory(rootdir, "All Packages");
	e = calloc(1, sizeof(struct directory_entry));
	e->next = d->list; d->list = e; e->content.pkg = p;
	directory_entry_count++;

	p->is_installed = is_installed;
	p->will_be_installed = is_installed;

	while ( fgets(line, 160, f) != NULL ) {
		if ( (line_length = strlen(line)) < 3 ) continue;
		if ( line[line_length-1] == '\n' ) line[--line_length] = 0;

		if ( !strncmp(line, "[V] ", 4) ) {
			p->version = malloc(line_length-3);
			strcpy(p->version, line+4);
			continue;
		}

		if ( !strncmp(line, "[C] ", 4) ) {
			t=strtok(line, " ");
			while ( (t=strtok(NULL, " ")) != NULL ) {
				d = get_create_directory(rootdir, t);
				e = calloc(1, sizeof(struct directory_entry));
				e->next = d->list; d->list = e;
				e->content.pkg = p;
				directory_entry_count++;
			}
			continue;
		}
	}
	fclose(f);

	snprintf(line, 160, "%s/pkgs/%s-%s.gem", config, p->name, p->version);
	if ( memdb_get(&files_on_disks, line) ) {
		p->disk_number = atoi(memdb_search_result);
	} else {
		snprintf(line, 160, "%s/pkgs/%s.gem", config, p->name);
		if ( memdb_get(&files_on_disks, line) )
			p->disk_number = atoi(memdb_search_result);
	}

	return 0;
}

void sort_directory(struct directory *d)
{
	struct directory_entry *e, **l;
	int size=0, i=0, keep_running=1;
	char *n1, *n2;

	/* Create array for easier sorting */
	for (e = d->list; e != NULL; e = e->next) size++;
	l = malloc( sizeof(struct directory_entry*) * size );
	for (e = d->list; e != NULL; e = e->next) l[i++] = e;

	/* Do normal bubble-sort */
	while ( keep_running )
	  for (keep_running=i=0; i < size-1; i++) {
		if ( l[i]->content_is_subdir )
			n1 = l[i]->content.dir->name;
		else
			n1 = l[i]->content.pkg->name;

		if ( l[i+1]->content_is_subdir )
			n2 = l[i+1]->content.dir->name;
		else
			n2 = l[i+1]->content.pkg->name;

		if ( strcmp(n1, n2) > 0 ) {
			e = l[i]; l[i] = l[i+1]; l[i+1] = e;
			keep_running=1;
		}
	  }

	/* Re-create linked-list */
	d->list = l[0];
	for (i=0; i < size-1; i++) l[i]->next = l[i+1];
	l[i]->next = NULL;

	/* Free helper-array */
	free(l);
}

int read_packages()
{
	struct dirent **namelist;
	struct directory *d;
	char filename1[PATH_MAX];
	char filename2[PATH_MAX];
	char descdir[PATH_MAX];
	char varadmdir[PATH_MAX];
	char indexfile[PATH_MAX];
	FILE *f;
	int i, j;

	snprintf(descdir, PATH_MAX, "%s/%s/info/descs", sourcedir, config);
	snprintf(indexfile, PATH_MAX, "%s/index.txt", sourcedir);
	snprintf(varadmdir, PATH_MAX, "%s/var/adm", targetdir);

	memdb_init(&files_on_disks);
	if ( (f = fopen(indexfile, "r")) != NULL ) {
		char filename[200], pkgdir[200], disk[200], line[160];

		snprintf(pkgdir, 200, "%s/pkgs/", config);

		while ( fgets(line, 160, f) != NULL ) {
			sscanf(line, "disk%s %s", disk, filename);
			if ( !strncmp(filename, pkgdir, strlen(pkgdir)) )
				memdb_put(&files_on_disks, filename, disk);
			if ( atoi(disk) > max_disk_number )
				max_disk_number = atoi(disk);
		}
		fclose(f);
	}

	j = scandir(descdir, &namelist, 0, alphasort);
	if (j < 0) {
		perror("scandir");
		return -1;
	} else {
		for (i=0; i<j; i++) {
			if ( namelist[i]->d_name[0] != '.' &&
			     strcmp(namelist[i]->d_name, "TRANS.TBL") ) {
				snprintf(filename1, PATH_MAX, "%s/%s",
				         descdir, namelist[i]->d_name);
				snprintf(filename2, PATH_MAX, "%s/flists/%s",
				         varadmdir, namelist[i]->d_name);
				read_package(namelist[i]->d_name, filename1,
				             access(filename2, F_OK) == 0);
			} else
				free(namelist[i]);
		}
		free(namelist);
	}

	for (d = directories; d != NULL; d = d->next) sort_directory(d);

	memdb_free(&files_on_disks);
	return 0;
}

